@@ -33,9 +33,13 @@ int main (int argc, char **argv)
 {
         GList *devices, *l;
 	gboolean has_touchpad, has_touchscreen;
+        gboolean has_mouse;
 
 	gtk_init (&argc, &argv);
 
+	has_mouse = mouse_is_present ();
+	g_print ("Has mouse:\t\t\t\t%s\n", has_mouse ? "yes" : "no");
+
 	has_touchpad = touchpad_is_present ();
 	g_print ("Has touchpad:\t\t\t\t%s\n", has_touchpad ? "yes" : "no");
 
