@@ -92,7 +92,7 @@ class GSDTestCase(X11SessionTestCase):
         klass.start_session()
         klass.start_monitor()
 
-        klass.settings_session = Gio.Settings('org.gnome.desktop.session')
+        klass.settings_session = Gio.Settings(schema_id='org.gnome.desktop.session')
 
     @classmethod
     def tearDownClass(klass):
